import 'dart:async';

import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'types/gestures.dart';

/// The interface that implementations of in_app_purchase must implement.
///
/// Platform implementations should extend this class rather than implement it as `contentsquare`
/// does not consider newly added methods to be breaking changes. Extending this class
/// (using `extends`) ensures that the subclass will get the default implementation, while
/// platform implementations that `implements` this interface will be broken by newly added
/// [ContentsquarePlatform] methods.
abstract class ContentsquarePlatform extends PlatformInterface {
  /// Constructs a ContentsquarePlatform.
  ContentsquarePlatform() : super(token: _token);

  static final Object _token = Object();

  /// The instance of [ContentsquarePlatform] to use.
  ///
  /// Must be set before accessing.
  static ContentsquarePlatform get instance => _instance;

  /// Platform-specific plugins should set this with their own platform-specific
  /// class that extends [ContentsquarePlatform] when they register themselves.
  static set instance(ContentsquarePlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  // Should only be accessed after setter is called.
  static late ContentsquarePlatform _instance;

  /// Tracks a screen view with the given name.
  /// You should use this method to track every screen viewed by a user.
  ///
  /// See also
  /// [Screen views in developer documentation](https://docs.contentsquare.com/flutter/#track-screens)
  Future<void> send(String screenName) async => throw UnimplementedError();

  /// Tracks the given transaction.
  /// Use this method to track purchases made by a user.
  ///
  /// You can use an use the [Currency.toStr] to get the String reprensentation for the `currency` parameter.
  Future<void> sendTransaction(double price, String currency,
          [String? transactionId]) async =>
      throw UnimplementedError();

  /// Tracks a dynamic var.
  ///
  /// You should use either the `stringValue` parameter or the `intValue` parameter
  Future<void> sendDynamicVar(String key,
          {String? stringValue, int? intValue}) async =>
      throw UnimplementedError();

  /// Opts the current user's device into being tracked.
  ///
  /// When this method is called tracking starts immediately.
  Future<void> optIn() async => throw UnimplementedError();

  /// Opts the current user's device out from being tracked.
  ///
  /// Immediately stops tracking the current device. The device will never be tracked again, unless the app is
  /// reinstalled, or you call the `optIn` method. In such case, the user will be tracked with a new user id and
  /// won't be linked to any previous activity.
  Future<void> optOut() async => throw UnimplementedError();

  /// Stops tracking current runtime and wipes user data.
  ///
  /// Any future tracked activity from the current user will have a new user id, and won't be linked to any past
  /// activity from this user. Tracking will resume on next app launch.
  Future<void> forgetMe() async => throw UnimplementedError();

  /// The anonymous id used by Contentsquare's SDK to track the current user.
  Future<String?> getUserId() async => throw UnimplementedError();

  /// Stops any further tracking for the current runtime.
  ///
  /// Call this method if you need to pause tracking on some particularly sensitive part of your app. You can resume
  /// tracking at any point by calling [resumeTracking]. Tracking will also resume on next app launch.
  Future<void> stopTracking() async => throw UnimplementedError();

  /// Resumes tracking.
  ///
  /// Call this method when you want to resume tracking after having called `stopTracking`.
  Future<void> resumeTracking() async => throw UnimplementedError();

  /// Send touch event.
  ///
  /// Call this method when you want to track a touch event
  Future<void> sendTouchEvent(
          {required ContentsquareGesture type,
          required int version,
          required TouchEventData data}) async =>
      throw UnimplementedError();
}
